@@ -22,11 +22,11 @@ setMethod("show", "boost_family", function(object) {
 })
 
 Family <- function(ngradient, loss = NULL, risk = NULL,
-                   offset = function(y, w) 
-                       optimize(risk, interval = range(y), y = y, w = w)$minimum, 
+                   offset = function(y, w)
+                       optimize(risk, interval = range(y), y = y, w = w)$minimum,
                    check_y = function(y) y,
-                   weights = c("any", "none", "zeroone", "case"), 
-                   nuisance = function() return(NULL),
+                   weights = c("any", "none", "zeroone", "case"),
+                   nuisance = function() return(NA),
                    name = "user-specified", fW = NULL, response = function(f) NA)
 {
 
@@ -42,7 +42,7 @@ Family <- function(ngradient, loss = NULL, risk = NULL,
     }
     weights <- match.arg(weights)
     check_w <- function(w) {
-        switch(weights, 
+        switch(weights,
             "any" = TRUE,
             "none" = isTRUE(all.equal(unique(w), 1)),
             "zeroone" = isTRUE(all.equal(unique(w + abs(w - 1)), 1)),
@@ -53,7 +53,7 @@ Family <- function(ngradient, loss = NULL, risk = NULL,
                weights = check_w, name = name, charloss = charloss)
     if (!is.null(fW))
         RET <- new("boost_family_glm", ngradient = ngradient, nuisance = nuisance,
-                   risk = risk, offset = offset, fW = fW, check_y = check_y, 
+                   risk = risk, offset = offset, fW = fW, check_y = check_y,
                    weights = check_w, name = name, response = response,
                    charloss= charloss)
     RET
@@ -73,6 +73,7 @@ Gaussian <- function()
            name = "Squared Error (Regression)",
            fW = function(f) return(rep(1, length = length(f))),
            response = function(f) f)
+
 GaussReg <- Gaussian
 
 ### Gaussian (-1 / 1 Binary Classification)
@@ -121,7 +122,7 @@ Binomial <- function()
            check_y = function(y) {
                if (!is.factor(y))
                    stop("response is not a factor but ",
-                           sQuote("family = Binomial()"))  
+                           sQuote("family = Binomial()"))
                if (nlevels(y) != 2)
                    stop("response is not a factor at two levels but ",
                            sQuote("family = Binomial()"))
@@ -167,8 +168,8 @@ Huber <- function(d = NULL) {
                         sQuote("family = Huber()"))
                y
            },
-           name = paste("Huber Error", 
-               ifelse(is.null(d), "(with adaptive d)", 
+           name = paste("Huber Error",
+               ifelse(is.null(d), "(with adaptive d)",
                                   paste("(with d = ", dtxt, ")", sep = ""))))
 }
 
@@ -183,7 +184,7 @@ AdaExp <- function()
            check_y = function(y) {
                if (!is.factor(y))
                    stop("response is not a factor but ",
-                           sQuote("family = AdaExp()"))  
+                           sQuote("family = AdaExp()"))
                if (nlevels(y) != 2)
                    stop("response is not a factor at two levels but ",
                            sQuote("family = AdaExp()"))
@@ -220,12 +221,14 @@ CoxPH <- function() {
                if (length(f) == 1)
                    f <- rep(f, length(time))
                storage.mode(f) <- "double"
+               w[is.na(f)] <- 0.0
+               f[is.na(f)] <- 0.0
                .Call("ngradientCoxPLik", time, event, f, w, package = "mboost")
            },
            risk = risk <- function(y, f, w = 1) -sum(plloss(y, f, w), na.rm = TRUE),
-           offset = function(y, w) 
-               optimize(risk, interval = c(0, max(y[,1], na.rm = TRUE)), 
-                        y = y, w = w)$minimum, 
+           offset = function(y, w)
+               optimize(risk, interval = c(0, max(y[,1], na.rm = TRUE)),
+                        y = y, w = w)$minimum,
            check_y = function(y) {
                if (!inherits(y, "Surv"))
                    stop("response is not an object of class ", sQuote("Surv"),
@@ -239,15 +242,15 @@ QuantReg <- function(tau = 0.5, qoffset = 0.5) {
     stopifnot(tau > 0 && tau < 1)
     stopifnot(qoffset > 0 && qoffset < 1)
     Family(
-        ngradient = function(y, f, w = 1) 
+        ngradient = function(y, f, w = 1)
             tau*((y - f) >= 0) - (1 - tau)*((y - f)<0) ,
         loss = function(y, f) tau*(y-f)*((y-f)>=0) - (1-tau)*(y-f)*((y-f)<0) ,
-        offset = function(y, w = rep(1, length(y))) 
+        offset = function(y, w = rep(1, length(y)))
             quantile(y[rep(1:length(y), w)], qoffset),
         weights = "case",
         check_y = function(y) {
             if (!is.numeric(y) || !is.null(dim(y)))
-                stop("response is not a numeric vector but ", 
+                stop("response is not a numeric vector but ",
                      sQuote("family = QuantReg()"))
             y
         },
@@ -262,24 +265,25 @@ NBinomial <- function(nuirange = c(0, 100)) {
            sigma * log(sigma) - sigma*log(exp(f) + sigma) + y * f -
            y * log(exp(f) + sigma))
 
-    riskS <- function(sigma, y, fit, w = 1) 
+    riskS <- function(sigma, y, fit, w = 1)
         sum(w * plloss(y = y, f = fit, sigma = sigma))
-    risk <- function(y, f, w = 1) 
+    risk <- function(y, f, w = 1)
        sum(w * plloss(y = y, f = f, sigma = sigma))
 
     ngradient <- function(y, f, w = 1) {
-        sigma <<- optimize(riskS, interval = nuirange, 
+        sigma <<- optimize(riskS, interval = nuirange,
                            y = y, fit = f, w = w)$minimum
         y - (y + sigma)/(exp(f) + sigma) * exp(f)
     }
-	
+
     Family(ngradient = ngradient, risk = risk,
            check_y = function(y) {
                stopifnot(all.equal(unique(y - floor(y)), 0))
                y
            },
            nuisance = function() return(sigma),
-           name = "Negative Negative-Binomial Likelihood")
+           name = "Negative Negative-Binomial Likelihood",
+           response = function(f) exp(f))
 }
 
 PropOdds <- function(nuirange = c(-0.5, -1), offrange = c(-5, 5)) {
@@ -294,9 +298,9 @@ PropOdds <- function(nuirange = c(-0.5, -1), offrange = c(-5, 5)) {
         if (length(f) == 1) f <- rep(f, length(y))
         tmp <- lapply(1:(length(sigma) + 1), function(i) {
             if (i == 1) return(1 + exp(f - sigma[i]))
-            if (i == (length(sigma) + 1)) 
+            if (i == (length(sigma) + 1))
                 return(1 - 1/(1 + exp(f - sigma[i - 1])))
-            return(1 / (1 + exp(f - sigma[i])) -  
+            return(1 / (1 + exp(f - sigma[i])) -
                    1 / (1 + exp(f - sigma[i - 1])))
         })
         loss <- log(tmp[[1]]) * (y == levels(y)[1])
@@ -305,21 +309,21 @@ PropOdds <- function(nuirange = c(-0.5, -1), offrange = c(-5, 5)) {
         return(loss)
     }
 
-    riskS <- function(delta, y, fit, w = 1) 
+    riskS <- function(delta, y, fit, w = 1)
         sum(w * plloss(y = y, f = fit, sigma = d2s(delta)))
-    risk <- function(y, f, w = 1) 
+    risk <- function(y, f, w = 1)
         sum(w * plloss(y = y, f = f, sigma = sigma))
 
     ngradient <- function(y, f, w = 1) {
-        delta <<- optim(par = delta, fn = riskS, y = y, 
+        delta <<- optim(par = delta, fn = riskS, y = y,
                         fit = f, w = w, method = "BFGS")$par
         sigma <<- d2s(delta)
         if (length(f) == 1) f <- rep(f, length(y))
         ng <- sapply(1:(length(sigma) + 1), function(i) {
             if (i > 1 & i < (length(sigma) + 1)) {
-                ret <- (1 - exp(2 * f - sigma[i - 1] - sigma[i])) / 
-                   (1 + exp(f - sigma[i - 1]) + 
-                        exp(f - sigma[i]) + 
+                ret <- (1 - exp(2 * f - sigma[i - 1] - sigma[i])) /
+                   (1 + exp(f - sigma[i - 1]) +
+                        exp(f - sigma[i]) +
                         exp(2 * f - sigma[i - 1] - sigma[i]))
             } else {
                 if (i == 1) {
@@ -334,13 +338,13 @@ PropOdds <- function(nuirange = c(-0.5, -1), offrange = c(-5, 5)) {
     }
 
     offset <- function(y, w = 1) {
-        delta <<- seq(from = nuirange[1], to = nuirange[2], 
+        delta <<- seq(from = nuirange[1], to = nuirange[2],
                       length = nlevels(y) - 1)
         sigma <<- d2s(delta)
         optimize(risk, interval = offrange, y = y, w = w)$minimum
     }
 
-    Family(ngradient = ngradient, 
+    Family(ngradient = ngradient,
            risk = risk, offset = offset,
            check_y = function(y) {
                stopifnot(is.ordered(y))
@@ -352,7 +356,7 @@ PropOdds <- function(nuirange = c(-0.5, -1), offrange = c(-5, 5)) {
                    if (i == 1) return(1 / (1 + exp(f - sigma[i])))
                    if (i == (length(sigma) + 1))
                        return(1 - 1/(1 + exp(f - sigma[i - 1])))
-                   return(1 / (1 + exp(f - sigma[i])) -  
+                   return(1 / (1 + exp(f - sigma[i])) -
                        1 / (1 + exp(f - sigma[i - 1])))
                    })
                ret
@@ -375,23 +379,23 @@ Weibull <- function(nuirange = c(0, 100)) {
         - Sevent * log(fw(eta) / sigma) - (1 - Sevent) * log(Sw(eta))
         }
 
-    riskS <- function(sigma, y, fit, w = 1) 
+    riskS <- function(sigma, y, fit, w = 1)
         sum(w * plloss(y = y, f = fit, sigma = sigma))
-    risk <- function(y, f, w = 1) 
+    risk <- function(y, f, w = 1)
        sum(w * plloss(y = y, f = f, sigma = sigma))
 
     ngradient <- function(y, f, w = 1) {
-        sigma <<- optimize(riskS, interval = nuirange, 
-                           y = y, fit = f, w = w)$minimum             
+        sigma <<- optimize(riskS, interval = nuirange,
+                           y = y, fit = f, w = w)$minimum
         lnt <- log(y[,1])
         event <- y[,2]
         eta <- (lnt - f) / sigma
         (event * (exp(eta) - 1) + (1 - event) * exp(eta)) / sigma
         }
-	
+
     Family(ngradient = ngradient, risk = risk,
-           offset = function(y, w) 
-               optimize(risk, interval = c(0, max(y[,1], na.rm = TRUE)), 
+           offset = function(y, w)
+               optimize(risk, interval = c(0, max(y[,1], na.rm = TRUE)),
                         y = y, w = w)$minimum,
            check_y = function(y) {
                if (!inherits(y, "Surv"))
@@ -400,7 +404,8 @@ Weibull <- function(nuirange = c(0, 100)) {
                y
            },
            nuisance = function() return(sigma),
-           name = "Negative Weibull Likelihood")
+           name = "Negative Weibull Likelihood",
+           response = function(f) exp(f))
 }
 
 Loglog <- function(nuirange = c(0, 100)) {
@@ -419,24 +424,24 @@ Loglog <- function(nuirange = c(0, 100)) {
         - Sevent * log(fw(eta) / sigma) - (1 - Sevent) * log(Sw(eta))
         }
 
-    riskS <- function(sigma, y, fit, w = 1) 
+    riskS <- function(sigma, y, fit, w = 1)
         sum(w * plloss(y = y, f = fit, sigma = sigma))
-    risk <- function(y, f, w = 1) 
+    risk <- function(y, f, w = 1)
        sum(w * plloss(y = y, f = f, sigma = sigma))
 
     ngradient <- function(y, f, w = 1) {
-        sigma <<- optimize(riskS, interval = nuirange, 
-                           y = y, fit = f, w = w)$minimum   
+        sigma <<- optimize(riskS, interval = nuirange,
+                           y = y, fit = f, w = w)$minimum
         lnt <- log(y[,1])
         event <- y[,2]
         eta <- (lnt - f) / sigma
         nom <- (exp(-eta) + 1)
         (event * (2/nom - 1) + (1 - event) / nom) / sigma
         }
-	
+
     Family(ngradient = ngradient, risk = risk,
-           offset = function(y, w) 
-               optimize(risk, interval = c(0, max(y[,1], na.rm = TRUE)), 
+           offset = function(y, w)
+               optimize(risk, interval = c(0, max(y[,1], na.rm = TRUE)),
                         y = y, w = w)$minimum,
            check_y = function(y) {
                if (!inherits(y, "Surv"))
@@ -445,7 +450,8 @@ Loglog <- function(nuirange = c(0, 100)) {
                y
            },
            nuisance = function() return(sigma),
-           name = "Negative Log Logistic Likelihood")
+           name = "Negative Log Logistic Likelihood",
+           response = function(f) exp(f))
 }
 
 Lognormal <- function(nuirange = c(0, 100)) {
@@ -464,23 +470,23 @@ Lognormal <- function(nuirange = c(0, 100)) {
         - Sevent * log(fw(eta) / sigma) - (1 - Sevent) * log(Sw(eta))
         }
 
-    riskS <- function(sigma, y, fit, w = 1) 
+    riskS <- function(sigma, y, fit, w = 1)
         sum(w * plloss(y = y, f = fit, sigma = sigma))
-    risk <- function(y, f, w = 1) 
+    risk <- function(y, f, w = 1)
        sum(w * plloss(y = y, f = f, sigma = sigma))
 
     ngradient <- function(y, f, w = 1) {
-        sigma <<- optimize(riskS, interval = nuirange, 
-                           y = y, fit = f, w = w)$minimum  
+        sigma <<- optimize(riskS, interval = nuirange,
+                           y = y, fit = f, w = w)$minimum
         lnt <- log(y[,1])
         event <- y[,2]
         eta <- (lnt - f) / sigma
         (event * eta + (1 - event) * dnorm(eta) / (1 - pnorm(eta))) / sigma
         }
-	
+
     Family(ngradient = ngradient, risk = risk,
-           offset = function(y, w) 
-               optimize(risk, interval = c(0, max(y[,1], na.rm = TRUE)), 
+           offset = function(y, w)
+               optimize(risk, interval = c(0, max(y[,1], na.rm = TRUE)),
                         y = y, w = w)$minimum,
            check_y = function(y) {
                if (!inherits(y, "Surv"))
@@ -489,7 +495,8 @@ Lognormal <- function(nuirange = c(0, 100)) {
                y
            },
            nuisance = function() return(sigma),
-           name = "Negative Lognormal Likelihood")
+           name = "Negative Lognormal Likelihood",
+           response = function(f) exp(f))
 }
 
 ExpectReg <- function (tau = 0.5) {
@@ -497,16 +504,16 @@ ExpectReg <- function (tau = 0.5) {
     Family(
         ngradient = function(y, f, w = 1)
             2 * tau * (y - f) * ((y - f) > 0) - 2 * (1 - tau) *
-            (f - y) * ((y - f) < 0) + 0 * ((y - f) == 0), 
-        loss = function(y, f) tau * (y - f)^2 * 
+            (f - y) * ((y - f) < 0) + 0 * ((y - f) == 0),
+        loss = function(y, f) tau * (y - f)^2 *
             ((y - f) >= 0) + (1 - tau) * (y - f)^2 * ((y - f) < 0),
-        offset = function(y, w = rep(1, length(y))) 
-            mean(y[w == 1]), 
+        offset = function(y, w = rep(1, length(y)))
+            mean(y[w == 1]),
         check_y = function(y) {
             if (!is.numeric(y) || !is.null(dim(y)))
                 stop("response is not a numeric vector but ",
                   sQuote("family = ExpectReg()"))
             y
-        }, 
+        },
         name = "Expectile Regression")
 }
